"""Class for generating counts tables from templates"""

import pathlib
import sys

import msgspec
import rich

from cumulus_library import BaseTableBuilder, base_utils, errors, study_manifest
from cumulus_library.builders.statistics_templates import counts_templates

# Defined here for easy overriding by tests
DEFAULT_MIN_SUBJECT = 10


# Counts can be driven by a workflow config. See docs/workflows/counts.md for more details
# on syntax and expectations.


class CountsWorkflowAnnotation(msgspec.Struct, forbid_unknown_fields=True):
    field: str
    join_table: str
    join_field: str
    columns: list[list[str]]
    alt_target: str | None = None


class CountsFilterColumn(msgspec.Struct, forbid_unknown_fields=True):
    name: str
    values: list[str]
    include_nulls: bool


class CountsWorkflowTable(msgspec.Struct, forbid_unknown_fields=True, omit_defaults=True):
    source_table: str
    table_cols: list

    description: str | None = None
    where_clauses: list[str] | None = None
    min_subject: int | None = None
    primary_id: str | None = None
    secondary_table: str | None = None
    secondary_cols: list[str] | None = None
    secondary_id: str | None = None
    alt_secondary_join_id: str | None = None
    annotation: CountsWorkflowAnnotation | None = None
    filter_cols: list[CountsFilterColumn] | None = None


class CountsWorkflow(msgspec.Struct, forbid_unknown_fields=True, omit_defaults=True):
    config_type: str
    tables: dict[str, CountsWorkflowTable]


class CountsBuilder(BaseTableBuilder):
    """Extends BaseTableBuilder for counts-related use cases"""

    def __init__(
        self,
        manifest: study_manifest.StudyManifest,
        *args,
        toml_config_path: pathlib.Path | None = None,
        **kwargs,
    ):
        super().__init__()

        self.study_prefix = manifest.get_study_prefix()
        if toml_config_path:
            try:
                with open(toml_config_path, "rb") as file:
                    file_bytes = file.read()
                    self._workflow_config = msgspec.to_builtins(
                        msgspec.toml.decode(file_bytes, type=CountsWorkflow)
                    )
            except msgspec.ValidationError as e:
                sys.exit(
                    f"The counts workflow at {toml_config_path!s} contains an unexpected param: \n"
                    f"{e}"
                )

            # for cases where we want to load a portion of a parseed dict into objects
            for table_name, contents in self._workflow_config["tables"].items():
                if annotation := contents.get("annotation"):
                    self._workflow_config["tables"][table_name]["annotation"] = (
                        counts_templates.CountAnnotation(**annotation)
                    )
                if filter_cols := contents.get("filter_cols"):
                    parsed_filters = []
                    for col in filter_cols:
                        parsed_filters.append(counts_templates.FilterColumn(**col))
                    self._workflow_config["tables"][table_name]["filter_cols"] = parsed_filters
        else:
            self._workflow_config = None

    def get_table_name(self, table_name: str, duration=None) -> str:
        """Convenience method for constructing table name

        :param table_name: table name to add after the study prefix
        :param duration: a time period reflecting the table binning strategy
        """
        if duration:
            return f"{self.study_prefix}__{table_name}_{duration}"
        else:
            return f"{self.study_prefix}__{table_name}"

    def get_where_clauses(
        self, clause: list | str | None = None, min_subject: int | None = None
    ) -> list[str]:
        """Convenience method for constructing arbitrary where clauses.

        :param clause: either a string or a list of sql where statements
        :param min_subject: if clause is none, the bin size for a cnt_subject_ref filter
            (deprecated, use count_[fhir_resource](min_subject) instead)
        """
        if min_subject is None:
            min_subject = DEFAULT_MIN_SUBJECT
        if clause is None:
            return [f"cnt_subject_ref >= {min_subject}"]
        elif isinstance(clause, str):
            return [clause]
        elif isinstance(clause, list):
            return clause
        else:
            raise errors.CountsBuilderError(f"get_where_clauses invalid clause {clause}")

    def get_count_query(
        self,
        table_name: str,
        source_table: str,
        table_cols: str | list[list[str] | counts_templates.CountColumn],
        *args,
        min_subject: int | None = None,
        where_clauses: list[str] | None = None,
        filter_resource: bool | None = False,
        primary_id: str = "subject_ref",
        secondary_id: str | None = None,
        alt_secondary_join_id: str | None = None,
        secondary_table: str | None = None,
        secondary_cols: list[str] = [],
        annotation: counts_templates.CountAnnotation | None = None,
        filter_cols: list[list[str]] | counts_templates.FilterColumn | None = None,
        **kwargs,
    ) -> str:
        """Generates a counts table using a template

        :param table_name: The name of the table to create. Must start with study prefix
        :param source_table: The table to create counts data from
        :param table_cols: The columns from the source table to add to the count table
        :keyword where_clauses: An array of where clauses to use for filtering the data
        :keyword min_subject: An integer setting the minimum bin size for inclusion
            (default: 10)
        :keyword primary_id: the FHIR ID to count by. If not present, uses subject_ref
        :keyword secondary_table: a table to join to the source table
        :keyword secondary_id: a second id to striate counts by. if secondary_table is
            also present, will be used to join with the source table, assuming that
            the same column exists in both tables
        :keyword alt_secondary_join_id: if present, uses this field instead of secondary_id
            for joining with the source table
        :keyword secondary_cols: the columns to include from the secondary table
        :keyword annotation: A CountsAnnotation object describing a table to use as
            a metadata annotation source
        :keyword filter_cols: a series of FilterColumns, or list formatted like
            ['column name', ('desired val 1', 'desired val 2'), True/False for including nulls],
            used to add filtering statements to a filter section of a query. If you also
            want these columns in your output, include them in in either table_cols or
            secondary_cols
        """
        if min_subject is None:
            min_subject = DEFAULT_MIN_SUBJECT

        if not table_name or not source_table or not table_cols:
            raise errors.CountsBuilderError(
                f"count_query missing required arguments. output table: {table_name}"
            )

        for key in kwargs:
            if key == "skip_status_filter":  # pragma: no cover
                # Deprecated as of v5.1.0
                rich.print(
                    "[yellow]CountsBuilder deprecation notice[/yellow]: the behavior of "
                    "'skip_status_filter' is now the default behavior and the argument is "
                    "considered deprecated.\n"
                    "Support for it may be removed in a future version."
                )
            elif key == "patient_link":  # pragma: no cover
                # Deprecated as of v6.0.0
                rich.print(
                    "[yellow]CountsBuilder deprecation notice[/yellow]: 'patient_link' has been "
                    "renamed to 'primary_id'. Future versions may remove support for the "
                    "patient_link argument."
                )
        return counts_templates.get_count_query(
            table_name,
            source_table,
            table_cols,
            *args,
            min_subject=min_subject,
            where_clauses=where_clauses,
            primary_id=primary_id,
            secondary_id=secondary_id,
            alt_secondary_join_id=alt_secondary_join_id,
            secondary_table=secondary_table,
            secondary_cols=secondary_cols,
            annotation=annotation,
            filter_status=len(filter_cols) if filter_cols else False,
            filter_cols=filter_cols,
            **kwargs,
        )

    def write_counts(
        self, config: base_utils.StudyConfig, manifest: study_manifest.StudyManifest, filepath: str
    ):
        """Convenience method for writing counts queries to disk

        :param filepath: path to file to write queries out to.
        """
        self.prepare_queries(config=config, manifest=manifest)
        self.comment_queries()
        self.write_queries(path=pathlib.Path(filepath))

    def prepare_queries(
        self,
        config: base_utils.StudyConfig,
        manifest: study_manifest.StudyManifest,
        *args,
        **kwargs,
    ):
        """Renders out count queries

        By default, prepare_queries will look for a workflow config to populate tables.
        In cases where you are subclassing to write a python counts builder, you
        should override this method with your own logic.

        """
        if not self._workflow_config:
            return
        for table_name, config in self._workflow_config["tables"].items():
            self.queries.append(
                self.get_count_query(
                    table_name=f"{manifest.get_formatted_study_prefix()}{table_name}", **config
                )
            )



import cumulus_library
from cumulus_library.template_sql import sql_utils


class CorePractitionerRoleBuilder(cumulus_library.BaseTableBuilder):
    display_text = "Creating PractitionerRole tables..."

    def prepare_queries(self, *args, config: cumulus_library.StudyConfig, **kwargs):
        code_sources = [
            sql_utils.CodeableConceptConfig(
                source_table="practitionerrole",
                column_hierarchy=[("code", list)],
                target_table="core__practitionerrole_dn_code",
            ),
            sql_utils.CodeableConceptConfig(
                source_table="practitionerrole",
                column_hierarchy=[("specialty", list)],
                target_table="core__practitionerrole_dn_specialty",
            ),
        ]
        self.queries += sql_utils.denormalize_complex_objects(
            config.db, code_sources, "PracticionerRole"
        )



"""Module for generating encounter codeableConcept table"""

import cumulus_library
from cumulus_library.studies.discovery import code_definitions
from cumulus_library.studies.discovery.discovery_templates import discovery_templates


class CodeUnionBuilder(cumulus_library.BaseTableBuilder):
    display_text = "Unioning code tables..."

    def prepare_queries(
        self,
        *args,
        config: cumulus_library.StudyConfig,
        **kwargs,
    ):
        """Constructs queries related to condition codeableConcept

        :param config: A study config object
        """
        query = discovery_templates.get_system_union(
            "discovery__code_sources", code_definitions.code_list
        )
        self.queries.append(query)



"""Holds enums used across more than one module"""

import enum


class LogStatuses(enum.Enum):
    DEBUG = "debug"
    ERROR = "error"
    FINISHED = "finished"
    INFO = "info"
    RESUMED = "resumed"
    STARTED = "started"
    WARN = "warn"


class ProtectedTables(enum.Enum):
    """Tables created by cumulus for persistence outside of study rebuilds"""

    STATISTICS = "lib_statistics"
    TRANSACTIONS = "lib_transactions"
    BUILD_SOURCE = "lib_build_source"


class ProtectedTableKeywords(enum.Enum):
    """Tables with a pattern like '_{keyword}_' are not manually dropped."""

    ETL = "etl"
    LIB = "lib"
    NLP = "nlp"


class ResourceTypes(enum.StrEnum):
    ALLERGEYINTOLERANCE = "allergyintolerance"
    CONDITION = "condition"
    DIAGNOSTICREPORT = "diagnosticreport"
    DOCUMENTREFERENCE = "documentreference"
    ENCOUNTER = "encounter"
    MEDICATION = "medication"
    MEDICATIONREFERENCE = "medicationreference"
    OBSERVATION = "observation"
    PATIENT = "patient"


class StatisticsTypes(enum.Enum):
    """A subset of workflows that create statistics sampling artifacts"""

    PSM = "psm"


class ManifestActions(enum.StrEnum):
    """The types of actions you can use in a manifest"""

    SERIAL = "build:serial"
    PARALLEL = "build:parallel"
    SUBMANIFEST = "submanifest"
    COUNTS = "export:counts"
    ANNOTATED = "export:annotated_counts"
    FLAT = "export:flat"
    META = "export:meta"



"""Class for loading study configuration data from manifest.toml files"""

import dataclasses
import pathlib
import re
import shutil
import subprocess
import sys
import tomllib

import msgspec

from cumulus_library import enums, errors


@dataclasses.dataclass(kw_only=True)
class ManifestExport:
    name: str
    export_type: str


# These msgspec Structs define the expected formats for manifests & submanifests


class ManifestAction(msgspec.Struct, forbid_unknown_fields=True, omit_defaults=True):
    type: str | None = None
    description: str | None = None
    label: str | None = None
    files: list[str] | None = None
    tables: list[str] | None = None


class ManifestAdvancedOptions(msgspec.Struct, forbid_unknown_fields=True, omit_defaults=True):
    dedicated_schema: str | None = None
    dynamic_study_prefix: str | None = None


class ManifestConfig(msgspec.Struct, forbid_unknown_fields=True, omit_defaults=True):
    study_prefix: str | None = None
    description: str | None = None
    stages: dict[str, list[ManifestAction]] | None = None
    advanced_options: ManifestAdvancedOptions | None = None


class SubmanifestConfig(msgspec.Struct, forbid_unknown_fields=True, omit_defaults=True):
    actions: list[ManifestAction]


class StudyManifest:
    """Handles interactions with study directories and manifest files"""

    PREFIX_REGEX = re.compile(r"[a-zA-Z][a-zA-Z0-9_]*")
    VALID_ACTIONS = ("serial", "parallel", "submanifest")

    def __init__(
        self,
        study_path: pathlib.Path | str | None = None,
        data_path: pathlib.Path | str | None = None,
        *,
        options: dict[str, str] | None = None,
    ):
        """Instantiates a StudyManifest.

        Note: while you can supply a custom path to load an arbitrary toml file, only a file
        named 'manifest.toml' will be detected automatically. Other filenames can be used
        for experimentation, or for rendering out artifacts that can then be packaged as part
        of a study for distribution.

        :param study_path: A Path object pointing to the directory, or a toml path, optional
        :param data_path: A Path object pointing to the dir to save/load ancillary files from,
            optional
        :param options: Command line study-specific options for dynamic manifest values, optional
        """
        self._study_prefix = None
        self._study_path = None
        self.data_path = None
        self._study_config = {}
        self._has_stats = False
        if study_path is not None:
            self._load_study_manifest(pathlib.Path(study_path), options or {})
        if data_path is not None:
            self.data_path = pathlib.Path(data_path)

    def __repr__(self):  # pragma: no cover
        return str(self._study_config)

    def _read_toml(self, path, doc_type) -> dict:
        """Runs a toml file through a format checker and returns a python dict"""
        try:
            with open(path, "rb") as file:
                file_bytes = file.read()
                return msgspec.to_builtins(msgspec.toml.decode(file_bytes, type=doc_type))
        except FileNotFoundError as e:
            raise errors.StudyManifestFilesystemError(
                f"Missing or invalid manifest found at {path}"
            ) from e
        except (msgspec.ValidationError, msgspec.DecodeError) as e:
            raise errors.StudyManifestParsingError(
                f"Manifest formatting error at path {path}: {e!s}. \n"
                "You may be using an different version of a study manifest. See"
                "https://docs.smarthealthit.org/cumulus/library/creating-studies.html "
                "for more information about what fields are expected in a manifest."
            )

    def _validate_action(self, action, source):
        action_type = action.get("type")
        if action_type is not None and action_type not in [e.value for e in enums.ManifestActions]:
            raise errors.StudyManifestParsingError(
                f"Action type '{action_type}' in {source} is not a valid action.\n"
                f"Valid action types: {', '.join(enums.ManifestActions)}."
            )

    def _has_stats_workflows(self) -> bool:
        for stage in self._study_config.get("stages", {}).values():
            for action in stage:
                for file in action.get("files", []):
                    if file.endswith(".toml") or file.endswith(".workflow"):
                        with open(self._study_path / file, "rb") as f:
                            workflow_conf = tomllib.load(f)
                            if workflow_conf["config_type"] in [
                                x.value for x in enums.StatisticsTypes
                            ]:
                                return True
        return False

    def _format_action(self, action):
        """Handles optional param coercion for actions"""
        if action.get("type") is None:
            action["type"] = "build:serial"
        if action.get("label") is None and action.get("description") is not None:
            action["label"] = action["description"]
        return action

    ### toml parsing helper functions
    def _load_study_manifest(self, study_path: pathlib.Path, options: dict[str, str]) -> None:
        """Reads in a config object from a directory containing a manifest.toml

        :param study_path: A pathlib.Path object pointing to a study directory or a toml file
        :param options: Command line study-specific options (--option=A:B)
        :raises StudyManifestParsingError: the manifest.toml is malformed or missing.
        """
        if study_path.is_dir():
            study_path = study_path / "manifest.toml"
        elif not study_path.name.endswith(".toml"):  # pragma: no cover
            raise errors.CumulusLibraryError(f"{study_path} is not a valid toml file")
        config = self._read_toml(study_path, ManifestConfig)

        defined_stages = list(config.get("stages", {}).keys())

        if len(defined_stages) == 0:
            raise errors.StudyManifestParsingError(
                f"{study_path} does not contain any stage definitions.\n"
                "See https://docs.smarthealthit.org/cumulus/library/creating-studies.html "
                "for more details about creating manifests."
            )

        # If we're importing from submanifest, we'll do an inline swap out of the submanifest for
        # the actions defined in the submanifest.
        # This means we can assume everywhere else that we don't need to worry about submanifests.
        all_actions = []
        for stage in defined_stages:
            if stage == "all":
                raise errors.StudyManifestParsingError(
                    "'all' is a reserved word for stage name. Please select a different name."
                )
            actions = []
            for action in config["stages"][stage]:
                if action.get("type") == "submanifest":
                    for submanifest in action.get("files", []):
                        subconfig = self._read_toml(
                            study_path.parent / submanifest, SubmanifestConfig
                        )
                        for subaction in subconfig.get("actions"):
                            self._validate_action(subaction, study_path.parent / submanifest)
                            actions.append(self._format_action(subaction))
                            all_actions.append(subaction)
                else:
                    self._validate_action(action, study_path)
                    actions.append(self._format_action(action))
                    all_actions.append(action)
            config["stages"][stage] = actions

        # if there isn't a default stage, we'll assume that we should run the first one
        if "default" not in defined_stages:
            config["stages"]["default"] = config["stages"][defined_stages[0]]

        config["stages"]["all"] = all_actions

        # We'll set these to class vars now so we can reuse the class getters
        # to finish setup and validation
        self._study_config = config
        self._study_path = study_path.parent

        if dynamic_study_prefix := config.get("advanced_options", {}).get("dynamic_study_prefix"):
            self._study_prefix = self._run_dynamic_script(dynamic_study_prefix, options)
        elif config.get("study_prefix") and isinstance(config["study_prefix"], str):
            self._study_prefix = config["study_prefix"]

        if not self._study_prefix or not re.fullmatch(self.PREFIX_REGEX, self._study_prefix):
            raise errors.StudyManifestParsingError(f"Invalid prefix in manifest at {study_path}")
        self._study_prefix = self._study_prefix.lower()

        # Finally, let's see if we're using a sampling workflow

        self._has_stats = self._has_stats_workflows()

    def get_study_prefix(self) -> str | None:
        """Reads the name of a study prefix from the in-memory study config

        :returns: A string of the prefix in the manifest, or None if not found
        """
        return self._study_prefix

    def get_formatted_study_prefix(self) -> str | None:
        """Returns the appropriately formatted value for a study prefix"""
        if dedicated := self._study_config.get("advanced_options", {}).get("dedicated_schema"):
            return f"{dedicated}."
        return f"{self._study_prefix}__"

    def get_dedicated_schema(self) -> str | None:
        """Reads the contents of the dedicated schema in the options dict

        :returns: A dictionary of objects, or None if not found
        """
        options = self._study_config.get("advanced_options", {})
        return options.get("dedicated_schema")

    def get_stages(self) -> list:
        """Returns the names of all stages defined in the manifest"""
        return list(self._study_config.get("stages", {}).keys())

    def get_stage(self, stage) -> list:
        """Returns the contents of the specified stage"""
        return self._study_config.get("stages", {}).get(stage, [])

    def get_file_list(
        self,
        stage_name: str = "default",
        continue_from: str | None = None,
    ) -> list[str | dict] | None:
        """Reads the contents of the file_config array or dict from the manifest
        :param stage_name: the stage_name to get the files of ('default' if not specified)
        :continue_from: a specific file to pick up a build from.
        :returns: An array of files from the manifest
        :raises StudyManifestParsingError: If the file in continue_from does not exist
        """
        files = []
        found_continue_point = False
        stage = self.get_stage(stage_name)
        for action in stage:
            if not action.get("type", "").startswith("build:"):
                continue
            if continue_from and not found_continue_point:
                file_stems = [x.split(".", 1)[0] for x in action["files"]]
                if continue_from.split(".", 1)[0] in file_stems:
                    found_continue_point = True
                    if action.get("type") == enums.ManifestActions.PARALLEL.value:
                        files = files + action["files"]
                    else:
                        pos = file_stems.index(continue_from.split(".", 1)[0])
                        files = files + action["files"][pos:]
            else:
                files = files + action["files"]
        if continue_from and len(files) == 0:
            raise errors.StudyManifestParsingError(f"No files matching '{continue_from}' found")
        return files

    def get_export_table_list(self, stage_name: str = "default") -> list[ManifestExport] | None:
        """Reads the exportable tables from the manifest

        :param stage_name: if present, just search the stages defined in the stage_name list
        :returns: An array of tuples (table, export type) to export from the manifest,
        or None if not found.
        """
        export_table_list = []
        stage = self.get_stage(stage_name)

        for action in stage:
            if not action.get("type", "").startswith("export:"):
                continue
            export_type = action["type"].split(":")[1]
            if export_type == "counts":
                # the aggregator is looking for the cube keyword, so we'll swap it out
                export_type = "cube"
            for table in action.get("tables", []):
                if table.startswith(f"{self.get_study_prefix()}__"):
                    export_table_list.append(ManifestExport(name=table, export_type=export_type))
                elif "__" in table:  # has a prefix, just the wrong one
                    raise errors.StudyManifestParsingError(
                        f"{table} in export list does not start with prefix "
                        f"{self.get_study_prefix()}__ - check your manifest file."
                    )
                else:
                    # Add the prefix for them (helpful in dynamic prefix cases where the prefix
                    # is not known ahead of time)
                    export_table_list.append(
                        ManifestExport(
                            name=f"{self.get_study_prefix()}__{table}", export_type=export_type
                        )
                    )

        found_name = set()
        for export in export_table_list:
            if export.name in found_name:
                raise errors.StudyManifestParsingError(
                    f"Table {export.name} is defined in multiple export sections. Tables should "
                    "only be defined in one export section in the study manifest"
                )
            else:
                found_name.add(export.name)
        return export_table_list

    def has_stats(self):
        return self._has_stats

    def get_all_files(self, file_type: str, stage_name: str = "default"):
        """Convenience method for getting files of a type from a manifest"""
        files = self.get_file_list(stage_name)
        return [file for file in files if file.endswith(file_type)]

    def get_all_generators(self, stage_name: str = "default") -> list[str]:
        """Convenience method for getting builder-based files"""
        return self.get_all_files(".py", stage_name)

    def get_all_workflows(self, stage_name: str = "default") -> list[str]:
        """Convenience method for getting workflow config files"""
        return self.get_all_files(".toml", stage_name)

    def get_prefix_with_seperator(self) -> str:
        """Convenience method for getting the appropriate prefix for tables"""
        if dedicated := self.get_dedicated_schema():
            return f"{dedicated}."
        return f"{self.get_study_prefix()}__"

    def copy_manifest(self, path: pathlib.Path):
        """Writes a copy of the manifest to the provided path.

        The intent of this is to get a copy of the manifest at runtime and copy it
        for purposes of exporting as part of an upload.
        """
        (path / self._study_prefix).mkdir(exist_ok=True, parents=True)
        shutil.copy(
            self._study_path / "manifest.toml", path / f"{self._study_prefix}/manifest.toml"
        )

    def write_manifest(self, path: pathlib.Path):
        if path.is_dir():
            path = path / "manifest.toml"
        path.parent.mkdir(exist_ok=True, parents=True)
        with open(path, "wb") as f:
            # we'll remove the all key we auto created before writing out a copy
            config = self._study_config
            config["stages"].pop("all", None)
            f.write(msgspec.toml.encode(ManifestConfig(self._study_config)))

    ### Dynamic Python code support

    def _run_dynamic_script(self, filename: str, options: dict[str, str]) -> str:
        if not sys.executable:
            raise RuntimeError("Unknown Python executable, dynamic manifest values not supported")

        full_path = f"{self._study_path}/{filename}"
        option_args = [f"--{key}={value}" for key, value in options.items()]
        result = subprocess.run(  # noqa: S603
            [sys.executable, full_path, *option_args],
            check=True,
            capture_output=True,
        )

        return result.stdout.decode("utf8").strip()



#!/usr/bin/env python3

"""Define schemas for the example_nlp study"""

import json
import os

import pydantic


class AgeMention(pydantic.BaseModel):
    has_mention: bool | None = pydantic.Field(None)
    spans: list[str] = pydantic.Field(default_factory=list, description="Supporting text spans")
    age: int | None = pydantic.Field(None, description="The age of the patient")


if __name__ == "__main__":
    basedir = os.path.dirname(__file__)

    with open(f"{basedir}/age.json", "w", encoding="utf8") as f:
        json.dump(AgeMention.model_json_schema(), f, indent=2)



"""A list of files/filename partials to be allowed to be uploaded to an aggregator

While a valid generation target, `.archive.` should never be part of this list, since
that explicitly contains line level data.
"""

ALLOWED_UPLOADS = [".cube.", ".annotated_cube.", ".flat.", ".meta.", "manifest.toml"]


"""ProtectedTableBuilder configurations that may need to be looked up elsewhere"""

TRANSACTIONS_COLS = ["study_name", "library_version", "status", "event_time", "message"]
TRANSACTION_COLS_TYPES = ["varchar", "varchar", "varchar", "timestamp", "varchar"]
# while it may seem redundant, study_name and view_name are included as a column for
# ease of constructing a view of multiple transaction tables
STATISTICS_COLS = [
    "study_name",
    "library_version",
    "table_type",
    "table_name",
    "view_name",
    "created_on",
]
STATISTICS_COLS_TYPES = [
    "varchar",
    "varchar",
    "varchar",
    "varchar",
    "varchar",
    "timestamp",
]
BUILD_SOURCE_COLS = ["stage", "name", "type"]
BUILD_SOURCE_COLS_ATHENA_TYPE = ["string", "string", "string"]
BUILD_SOURCE_COLS_SQL_TYPE = ["varchar", "varchar", "varchar"]
